import 'dart:async';
import 'dart:io' show Platform;
import 'package:flutter/services.dart';
import 'package:onesignal_flutter/src/debug.dart';
import 'package:onesignal_flutter/src/user.dart';
import 'package:onesignal_flutter/src/notifications.dart';
import 'package:onesignal_flutter/src/session.dart';
import 'package:onesignal_flutter/src/location.dart';
import 'package:onesignal_flutter/src/inappmessages.dart';

export 'src/defines.dart';
export 'src/pushsubscription.dart';
export 'src/subscription.dart';
export 'src/notification.dart';
export 'src/notifications.dart';
export 'src/inappmessage.dart';

class OneSignal {
  /// A singleton representing the OneSignal SDK.
  /// Note that the iOS and Android native libraries are static,
  /// so if you create multiple instances of OneSignal, they will
  /// mostly share the same state.
  static OneSignal shared = new OneSignal();
  static OneSignalDebug Debug = new OneSignalDebug();
  static OneSignalUser User = new OneSignalUser();
  static OneSignalNotifications Notifications = new OneSignalNotifications();
  static OneSignalSession Session = new OneSignalSession();
  static OneSignalLocation Location = new OneSignalLocation();
  static OneSignalInAppMessages InAppMessages = new OneSignalInAppMessages();

  // private channels used to bridge to ObjC/Java
  MethodChannel _channel = const MethodChannel('OneSignal');

  /// The initializer for OneSignal.
  ///
  /// The initializer accepts an [appId] which the developer can get
  /// from the OneSignal consoleas well as a dictonary of [launchOptions]
  void initialize(String appId) {
    _channel.invokeMethod('OneSignal#initialize', {'appId': appId});
    InAppMessages.lifecycleInit();
    User.pushSubscription.lifecycleInit();
    Notifications.lifecycleInit();
  }

  /// Login to OneSignal under the user identified by the [externalId] provided.
  ///
  /// The act of logging a user into the OneSignal SDK will switch the
  /// user context to that specific user.
  Future<void> login(String externalId) async {
    return await _channel
        .invokeMethod('OneSignal#login', {'externalId': externalId});
  }

  /// Login to OneSignal under the user identified by the [externalId] provided.
  ///
  /// The act of logging a user into the OneSignal SDK will switch the
  /// user context to that specific user.
  Future<void> loginWithJWT(String externalId, String jwt) async {
    if (Platform.isAndroid) {
      return await _channel.invokeMethod(
          'OneSignal#loginWithJWT', {'externalId': externalId, 'jwt': jwt});
    }
  }

  /// Logout the user previously logged in via [login]. The user property now
  ///
  /// references a new device-scoped user. A device-scoped user has no user identity
  /// that can later be retrieved, except through this device as long as the app
  /// remains installed and the app data is not cleared.
  Future<void> logout() async {
    return await _channel.invokeMethod('OneSignal#logout');
  }

  /// Indicates whether privacy consent has been granted.
  ///
  /// This field is only relevant when the application has
  /// opted into data privacy protections. See [requiresPrivacyConsent].
  Future<bool> getPrivacyConsent() async {
    var val = await _channel.invokeMethod("OneSignal#getPrivacyConsent");

    return val as bool;
  }

  /// Sets the whether or not privacy consent has been [granted]
  ///
  /// This field is only relevant when the application has
  /// opted into data privacy protections. See [requiresPrivacyConsent].
  Future<void> setPrivacyConsent(bool granted) async {
    return await _channel
        .invokeMethod("OneSignal#setPrivacyConsent", {'granted': granted});
  }

  /// A boolean value indicating if the OneSignal SDK is waiting for the
  /// user's consent before it can initialize (if you set the app to
  /// require the user's consent)
  Future<bool> requiresPrivacyConsent() async {
    var val = await _channel.invokeMethod("OneSignal#requiresPrivacyConsent");

    return val as bool;
  }

  /// Allows you to completely disable the SDK until your app calls the
  /// OneSignal.setPrivacyConsent(true) function. This is useful if you want
  /// to show a Terms and Conditions or privacy popup for GDPR.
  Future<void> setRequiresPrivacyConsent(bool require) async {
    return await _channel.invokeMethod(
        "OneSignal#setRequiresPrivacyConsent", {'required': require});
  }

  /// This method can be used to set if launch URLs should be opened in safari or
  /// within the application. Set to true to launch all notifications with a URL
  /// in the app instead of the default web browser. Make sure to call setLaunchURLsInApp
  /// before the initialize call.
  Future<void> setLaunchURLsInApp(bool launchUrlsInApp) async {
    if (Platform.isIOS) {
      return await _channel.invokeMethod(
          'OneSignal#setLaunchURLsInApp', {'launchUrlsInApp': launchUrlsInApp});
    }
  }

  /// Only applies to iOS
  /// Associates a temporary push token with an Activity ID on the OneSignal server.
  Future<void> enterLiveActivity(String activityId, String token) async {
    if (Platform.isIOS) {
      return await _channel.invokeMethod("OneSignal#enterLiveActivity",
          {'activityId': activityId, 'token': token});
    }
  }

  /// Only applies to iOS
  /// Deletes activityId associated temporary push token on the OneSignal server.
  Future<void> exitLiveActivity(String activityId) async {
    if (Platform.isIOS) {
      return await _channel.invokeMethod(
          "OneSignal#exitLiveActivity", {'activityId': activityId});
    }
  }
}
